from __future__ import annotations

from collections.abc import Callable, Sequence
import functools

from typing import TYPE_CHECKING

from prettyqt import core


if TYPE_CHECKING:
    from prettyqt import widgets


def is_descendent_of(
    indexes: list[core.ModelIndex] | core.QItemSelection | core.QItemSelectionRange,
    index: core.ModelIndex,
) -> bool:
    if not index.isValid():
        return False
    match indexes:
        case list():
            if index in indexes:
                return False
            while (index := index.parent()).isValid():
                if index in indexes:
                    return True
            return False
        case core.QItemSelection() | core.QItemSelectionRange():
            if indexes.contains(index):
                return False
            while (index := index.parent()).isValid():
                if indexes.contains(index):
                    return True
            return False
        case _:
            raise TypeError(indexes)


def find_root_index(index: core.ModelIndex) -> core.ModelIndex:
    while index.parent().isValid():
        index = index.parent()
    return index


def get_parent_indexes(index: core.ModelIndex) -> list[core.ModelIndex]:
    indexes = []
    while index.isValid():
        indexes.insert(0, index)
        index = index.parent()
    return indexes


def get_proxy_chain(model: core.QAbstractItemModel) -> list[core.QAbstractItemModel]:
    models = [model]
    while isinstance(model, core.QAbstractProxyModel):
        model = model.sourceModel()
        models.append(model)
    return models


def index_from_key(
    model: core.QAbstractItemModel,
    key_path: Sequence[tuple[int, int] | int],
    parent_index: core.ModelIndex,
) -> core.ModelIndex:
    if model is None:
        return core.ModelIndex()
    index = parent_index or core.ModelIndex()
    for key in key_path:
        key = (key, 0) if isinstance(key, int) else key
        index = model.index(*key, index)
    return index


def ci(
    index_is_valid: bool = False,
    do_not_use_parent: bool = False,
    parent_is_invalid: bool = False,
):
    def inner(fn: Callable) -> Callable:
        @functools.wraps(fn)
        def wrapper(
            ref: core.AbstractItemModelMixin, index: core.QModelIndex, *args, **kwargs
        ):
            if ref.check_index(
                index, index_is_valid, do_not_use_parent, parent_is_invalid
            ):
                return fn(ref, index, *args, **kwargs)
            else:
                raise TypeError("Invalid index")

        return wrapper

    return inner


def requires_model(fn: Callable) -> Callable:
    @functools.wraps(fn)
    def wrapper(ref: widgets.AbstractItemViewMixin, *args, **kwargs):
        if ref.model() is None:
            raise RuntimeError(f"Trying to call {fn.__name__} without a model set.")
        return fn(ref, *args, **kwargs)

    return wrapper



"""A rich set of classes to handle multimedia content."""

from __future__ import annotations

from prettyqt.qt.QtMultimedia import *  # noqa: F403

from .mediametadata import MediaMetaData
from .audioformat import AudioFormat
from .audiodevice import AudioDevice
from .cameraformat import CameraFormat
from .cameradevice import CameraDevice
from .camera import Camera
from .mediaplayer import MediaPlayer
from .screencapture import ScreenCapture
from .mediarecorder import MediaRecorder
from prettyqt.qt import QtMultimedia

QT_MODULE = QtMultimedia

__all__ = [
    "AudioFormat",
    "AudioDevice",
    "Camera",
    "CameraDevice",
    "CameraFormat",
    "MediaPlayer",
    "MediaRecorder",
    "ScreenCapture",
    "MediaMetaData",
]



from __future__ import annotations

import enum

from typing import Literal

from prettyqt.qt import QtCore
from prettyqt.utils import bidict, get_repr


TypeStr = Literal[
    "void",
    "bool",
    "int",
    "u_int",
    "double",
    "qchar",
    "qstring",
    "byte_array",
    "nullptr",
    "void_star",
    "long",
    "long_long",
    "short",
    "char",
    "char_16",
    "char_32",
    "u_long",
    "u_long_long",
    "u_short",
    "s_char",
    "u_char",
    "float",
    "float_16",
    "object_star",
    "cursor",
    "date",
    "size",
    "time",
    "variant_list",
    "polygon",
    "polygonf",
    "color",
    "color_space",
    "sizef",
    "rectf",
    "line",
    "text_length",
    "string_list",
    "variant_map",
    "variant_hash",
    "variant_pair",
    "icon",
    "pen",
    "linef",
    "text_format",
    "rect",
    "point",
    "url",
    "regular_expression",
    "datetime",
    "pointf",
    "palette",
    "font",
    "brush",
    "region",
    "bit_array",
    "image",
    "key_sequence",
    "size_policy",
    "pixmap",
    "locale",
    "bitmap",
    "transform",
    "matrix_4x4",
    "vector_2d",
    "vector_3d",
    "vector_4d",
    "quaternion",
    "easing_curve",
    "json_value",
    "json_object",
    "json_array",
    "json_document",
    "cbor_value",
    "cbor_array",
    "cbor_map",
    "cbor_simple_style",
    "model_index",
    "persistent_model_index",
    "uuid",
    "byte_array_list",
    "variant",
    "user",
    "unknown_type",
]

TYPE = bidict(
    void=QtCore.QMetaType.Type.Void,
    bool=QtCore.QMetaType.Type.Bool,
    int=QtCore.QMetaType.Type.Int,
    u_int=QtCore.QMetaType.Type.UInt,
    double=QtCore.QMetaType.Type.Double,
    qchar=QtCore.QMetaType.Type.QChar,
    qstring=QtCore.QMetaType.Type.QString,
    byte_array=QtCore.QMetaType.Type.QByteArray,
    # nullptr=QtCore.QMetaType.Type.Nullptr,
    void_star=QtCore.QMetaType.Type.VoidStar,
    long=QtCore.QMetaType.Type.Long,
    long_long=QtCore.QMetaType.Type.LongLong,
    short=QtCore.QMetaType.Type.Short,
    char=QtCore.QMetaType.Type.Char,
    # char_16=QtCore.QMetaType.Type.Char16,
    # char_32=QtCore.QMetaType.Type.Char32,
    u_long=QtCore.QMetaType.Type.ULong,
    u_long_long=QtCore.QMetaType.Type.ULongLong,
    u_short=QtCore.QMetaType.Type.UShort,
    s_char=QtCore.QMetaType.Type.SChar,
    u_char=QtCore.QMetaType.Type.UChar,
    float=QtCore.QMetaType.Type.Float,
    # float_16=QtCore.QMetaType.Type.Float16,
    object_star=QtCore.QMetaType.Type.QObjectStar,
    cursor=QtCore.QMetaType.Type.QCursor,
    date=QtCore.QMetaType.Type.QDate,
    size=QtCore.QMetaType.Type.QSize,
    time=QtCore.QMetaType.Type.QTime,
    variant_list=QtCore.QMetaType.Type.QVariantList,
    polygon=QtCore.QMetaType.Type.QPolygon,
    polygonf=QtCore.QMetaType.Type.QPolygonF,
    color=QtCore.QMetaType.Type.QColor,
    color_space=QtCore.QMetaType.Type.QColorSpace,
    sizef=QtCore.QMetaType.Type.QSizeF,
    rectf=QtCore.QMetaType.Type.QRectF,
    line=QtCore.QMetaType.Type.QLine,
    text_length=QtCore.QMetaType.Type.QTextLength,
    string_list=QtCore.QMetaType.Type.QStringList,
    variant_map=QtCore.QMetaType.Type.QVariantMap,
    variant_hash=QtCore.QMetaType.Type.QVariantHash,
    # variant_pair=QtCore.QMetaType.Type.QVariantPair,
    icon=QtCore.QMetaType.Type.QIcon,
    pen=QtCore.QMetaType.Type.QPen,
    linef=QtCore.QMetaType.Type.QLineF,
    text_format=QtCore.QMetaType.Type.QTextFormat,
    rect=QtCore.QMetaType.Type.QRect,
    point=QtCore.QMetaType.Type.QPoint,
    url=QtCore.QMetaType.Type.QUrl,
    regular_expression=QtCore.QMetaType.Type.QRegularExpression,
    datetime=QtCore.QMetaType.Type.QDateTime,
    pointf=QtCore.QMetaType.Type.QPointF,
    palette=QtCore.QMetaType.Type.QPalette,
    font=QtCore.QMetaType.Type.QFont,
    brush=QtCore.QMetaType.Type.QBrush,
    region=QtCore.QMetaType.Type.QRegion,
    bit_array=QtCore.QMetaType.Type.QBitArray,
    image=QtCore.QMetaType.Type.QImage,
    key_sequence=QtCore.QMetaType.Type.QKeySequence,
    size_policy=QtCore.QMetaType.Type.QSizePolicy,
    pixmap=QtCore.QMetaType.Type.QPixmap,
    locale=QtCore.QMetaType.Type.QLocale,
    bitmap=QtCore.QMetaType.Type.QBitmap,
    transform=QtCore.QMetaType.Type.QTransform,
    matrix_4x4=QtCore.QMetaType.Type.QMatrix4x4,
    vector_2d=QtCore.QMetaType.Type.QVector2D,
    vector_3d=QtCore.QMetaType.Type.QVector3D,
    vector_4d=QtCore.QMetaType.Type.QVector4D,
    quaternion=QtCore.QMetaType.Type.QQuaternion,
    easing_curve=QtCore.QMetaType.Type.QEasingCurve,
    json_value=QtCore.QMetaType.Type.QJsonValue,
    json_object=QtCore.QMetaType.Type.QJsonObject,
    json_array=QtCore.QMetaType.Type.QJsonArray,
    json_document=QtCore.QMetaType.Type.QJsonDocument,
    cbor_value=QtCore.QMetaType.Type.QCborValue,
    cbor_array=QtCore.QMetaType.Type.QCborArray,
    cbor_map=QtCore.QMetaType.Type.QCborMap,
    cbor_simple_style=QtCore.QMetaType.Type.QCborSimpleType,
    model_index=QtCore.QMetaType.Type.QModelIndex,
    persistent_model_index=QtCore.QMetaType.Type.QPersistentModelIndex,
    uuid=QtCore.QMetaType.Type.QUuid,
    byte_array_list=QtCore.QMetaType.Type.QByteArrayList,
    variant=QtCore.QMetaType.Type.QVariant,
    user=QtCore.QMetaType.Type.User,
    unknown_type=QtCore.QMetaType.Type.UnknownType,
)

TypeFlagStr = Literal[
    "needs_construction",
    "needs_copy_construction",
    "needs_move_construction",
    "needs_destruction",
    "relocatable_type",
    "is_enumeration",
    "is_unsigned_enumeration",
    "pointer_to_qobject",
    "is_pointer",
    "is_const",
]

TYPE_FLAG: bidict[TypeFlagStr, QtCore.QMetaType.TypeFlag] = bidict(
    needs_construction=QtCore.QMetaType.TypeFlag.NeedsConstruction,
    needs_copy_construction=QtCore.QMetaType.TypeFlag.NeedsCopyConstruction,
    needs_move_construction=QtCore.QMetaType.TypeFlag.NeedsMoveConstruction,
    needs_destruction=QtCore.QMetaType.TypeFlag.NeedsDestruction,
    relocatable_type=QtCore.QMetaType.TypeFlag.RelocatableType,
    is_enumeration=QtCore.QMetaType.TypeFlag.IsEnumeration,
    is_unsigned_enumeration=QtCore.QMetaType.TypeFlag.IsUnsignedEnumeration,
    pointer_to_qobject=QtCore.QMetaType.TypeFlag.PointerToQObject,
    is_pointer=QtCore.QMetaType.TypeFlag.IsPointer,
    is_const=QtCore.QMetaType.TypeFlag.IsConst,
)

TYPE_END_INDEX = 63

ENUM_START_INDEX = 65537
ENUM_END_INDEX = 66065


class MetaType(QtCore.QMetaType):
    """Manages named types in the meta-object system."""

    def __bool__(self):
        return self.isValid()

    def __repr__(self):
        return get_repr(self, self.get_name())

    @classmethod
    def get_regular_types(cls):
        return [
            meta for i in range(TYPE_END_INDEX + 1) if (meta := MetaType(i)).isValid()
        ]

    @classmethod
    def get_enum_types(cls):
        return [
            meta
            for i in range(ENUM_START_INDEX, ENUM_END_INDEX + 1)
            if (meta := MetaType(i)).isValid()
        ]

    def get_name(self) -> str | None:
        return name.decode() if isinstance(name := self.name(), bytes) else name

    def get_type_name(self) -> str:
        return TYPE.inverse[QtCore.QMetaType.Type(self.id())]

    def is_enumeration(self) -> bool:
        return bool(self.flags() & QtCore.QMetaType.TypeFlag.IsEnumeration)

    # def get_meta_object(self) -> core.MetaObject:  # apparently doesnt exist in bindings
    #     return core.MetaObject(self.metaObject())

    # @classmethod  # and this one gives deprecated message
    # def get_meta_object_for_type(cls, typ: int) -> core.MetaObject:
    #     return core.MetaObject(cls.metaObjectForType(typ))

    def get_type(self) -> type:
        meta_type = QtCore.QMetaType.Type(self.id())
        match meta_type:
            case QtCore.QMetaType.Type.Bool:
                return bool
            case QtCore.QMetaType.Type.Int | QtCore.QMetaType.Type.UInt:
                return int
            case QtCore.QMetaType.Type.Double | QtCore.QMetaType.Type.Float:
                return float
            case QtCore.QMetaType.Type.QChar | QtCore.QMetaType.Type.QString:
                return str
            case QtCore.QMetaType.Type.QByteArray:
                return bytes
            case QtCore.QMetaType.Type.QVariantList | QtCore.QMetaType.Type.QStringList:
                return list
            case QtCore.QMetaType.Type.QVariantMap:
                return dict
            case QtCore.QMetaType.Type.QSize:
                return QtCore.QSize
            case QtCore.QMetaType.Type.QSizeF:
                return QtCore.QSizeF
            case QtCore.QMetaType.Type.QTime:
                return QtCore.QTime
            case QtCore.QMetaType.Type.QDate:
                return QtCore.QDate
            case QtCore.QMetaType.Type.QDateTime:
                return QtCore.QDateTime
            case QtCore.QMetaType.Type.QRect:
                return QtCore.QRect
            case QtCore.QMetaType.Type.QRectF:
                return QtCore.QRectF
            case QtCore.QMetaType.Type.QLine:
                return QtCore.QLine
            case QtCore.QMetaType.Type.QLineF:
                return QtCore.QLineF
            case QtCore.QMetaType.Type.QPoint:
                return QtCore.QPoint
            case QtCore.QMetaType.Type.QPointF:
                return QtCore.QPointF
            case QtCore.QMetaType.Type.QRegularExpression:
                return QtCore.QRegularExpression
            case QtCore.QMetaType.Type.QLocale:
                return QtCore.QLocale
            case QtCore.QMetaType.Type.QUrl:
                return QtCore.QUrl
            case _ if self.is_enumeration():
                return enum.Enum

        from prettyqt.qt import QtGui

        match meta_type:
            case QtCore.QMetaType.Type.QPolygon:
                return QtGui.QPolygon
            case QtCore.QMetaType.Type.QPolygonF:
                return QtGui.QPolygonF
            case QtCore.QMetaType.Type.QTextLength:
                return QtGui.QTextLength
            case QtCore.QMetaType.Type.QRegion:
                return QtGui.QRegion
            case QtCore.QMetaType.Type.QPalette:
                return QtGui.QPalette
            case QtCore.QMetaType.Type.QColor:
                return QtGui.QColor
            case QtCore.QMetaType.Type.QPen:
                return QtGui.QPen
            case QtCore.QMetaType.Type.QFont:
                return QtGui.QFont
            case QtCore.QMetaType.Type.QBrush:
                return QtGui.QBrush
            case QtCore.QMetaType.Type.QImage:
                return QtGui.QImage
            case QtCore.QMetaType.Type.QPixmap:
                return QtGui.QPixmap
            case QtCore.QMetaType.Type.QTransform:
                return QtGui.QTransform
            case QtCore.QMetaType.Type.QKeySequence:
                return QtGui.QKeySequence
            case QtCore.QMetaType.Type.QVector3D:
                return QtGui.QVector3D
            case QtCore.QMetaType.Type.QVector4D:
                return QtGui.QVector4D
            case QtCore.QMetaType.Type.QCursor:
                return QtGui.QCursor
            case QtCore.QMetaType.Type.QIcon:
                return QtGui.QIcon
            case QtCore.QMetaType.Type.QVariant:
                return object
            case QtCore.QMetaType.Type.QEasingCurve:
                return QtCore.QEasingCurve
            case QtCore.QMetaType.Type.QObjectStar:
                return QtCore.QObject

        from prettyqt.qt import QtWidgets

        match meta_type:
            case QtCore.QMetaType.Type.QSizePolicy:
                return QtWidgets.QSizePolicy
            case _:
                return object


if __name__ == "__main__":
    metatype = MetaType(66532)
    print(metatype)



from __future__ import annotations

from typing import Literal

from prettyqt import bluetooth, core
from prettyqt.utils import bidict


SocketError = bluetooth.QBluetoothSocket.SocketError

SocketErrorStr = Literal[
    "unknown_socket",
    "no_socket",
    "host_not_found",
    "service_not_found",
    "network",
    "unsupported_protocol",
    "operation",
    "remote_host_closed",
    "missing_permissions",
]

SOCKET_ERROR: bidict[SocketErrorStr, SocketError] = bidict(
    unknown_socket=SocketError.UnknownSocketError,
    no_socket=SocketError.NoSocketError,
    host_not_found=SocketError.HostNotFoundError,
    service_not_found=SocketError.ServiceNotFoundError,
    network=SocketError.NetworkError,
    unsupported_protocol=SocketError.UnsupportedProtocolError,
    operation=SocketError.OperationError,
    remote_host_closed=SocketError.RemoteHostClosedError,
    missing_permissions=SocketError.MissingPermissionsError,
)


SocketState = bluetooth.QBluetoothSocket.SocketState
SocketStateStr = Literal[
    "unconnected",
    "service_lookup",
    "connecting",
    "connected",
    "bound",
    "closing",
    "listening",
]

SOCKET_STATE: bidict[SocketStateStr, SocketState] = bidict(
    unconnected=SocketState.UnconnectedState,
    service_lookup=SocketState.ServiceLookupState,
    connecting=SocketState.ConnectingState,
    connected=SocketState.ConnectedState,
    bound=SocketState.BoundState,
    closing=SocketState.ClosingState,
    listening=SocketState.ListeningState,
)


class BluetoothSocket(core.IODeviceMixin, bluetooth.QBluetoothSocket):
    """Enables connection to a Bluetooth device running a bluetooth server."""

    def __init__(
        self,
        protocol: bluetooth.bluetoothserviceinfo.ProtocolStr
        | bluetooth.QBluetoothServiceInfo.Protocol,
        parent: core.QObject | None = None,
    ):
        protocol = bluetooth.bluetoothserviceinfo.PROTOCOL.get_enum_value(protocol)
        super().__init__(protocol, parent)

    def get_error(self) -> SocketErrorStr:
        return SOCKET_ERROR.inverse[self.error()]

    def get_state(self) -> SocketStateStr:
        return SOCKET_STATE.inverse[self.state()]

    def get_local_address(self) -> bluetooth.BluetoothAddress:
        return bluetooth.BluetoothAddress(self.localAddress())

    def get_peer_address(self) -> bluetooth.BluetoothAddress:
        return bluetooth.BluetoothAddress(self.peerAddress())

    def get_socket_type(self) -> bluetooth.bluetoothserviceinfo.ProtocolStr:
        return bluetooth.bluetoothserviceinfo.PROTOCOL.inverse[self.socketType()]


if __name__ == "__main__":
    from prettyqt import core

    app = core.app()
    socket = BluetoothSocket("l2_cap")
    with app.debug_mode():
        app.exec()



from __future__ import annotations

import os
import pathlib

from typing import Literal

from prettyqt import core, widgets
from prettyqt.utils import bidict, datatypes


FileModeStr = Literal["existing_file", "existing_files", "any_file", "directory"]

FILE_MODE: bidict[FileModeStr, widgets.QFileDialog.FileMode] = bidict(
    existing_file=widgets.QFileDialog.FileMode.ExistingFile,
    existing_files=widgets.QFileDialog.FileMode.ExistingFiles,
    any_file=widgets.QFileDialog.FileMode.AnyFile,
    directory=widgets.QFileDialog.FileMode.Directory,
)

LabelStr = Literal["look_in", "filename", "filetype", "accept"]

LABEL: bidict[LabelStr, widgets.QFileDialog.DialogLabel] = bidict(
    look_in=widgets.QFileDialog.DialogLabel.LookIn,
    filename=widgets.QFileDialog.DialogLabel.FileName,
    filetype=widgets.QFileDialog.DialogLabel.FileType,
    accept=widgets.QFileDialog.DialogLabel.Accept,
    reject=widgets.QFileDialog.DialogLabel.Reject,
)

AcceptModeStr = Literal["save", "open"]

ACCEPT_MODE: bidict[AcceptModeStr, widgets.QFileDialog.AcceptMode] = bidict(
    save=widgets.QFileDialog.AcceptMode.AcceptSave,
    open=widgets.QFileDialog.AcceptMode.AcceptOpen,
)

ViewModeStr = Literal["detail", "list"]

VIEW_MODE: bidict[ViewModeStr, widgets.QFileDialog.ViewMode] = bidict(
    detail=widgets.QFileDialog.ViewMode.Detail, list=widgets.QFileDialog.ViewMode.List
)


class FileDialog(widgets.DialogMixin, widgets.QFileDialog):
    """Dialog that allow users to select files or directories."""

    def __init__(
        self,
        path: None | datatypes.PathType = None,
        mode: AcceptModeStr = "open",
        caption: str | None = None,
        path_id: str | None = None,
        extension_filter: dict | None = None,
        file_mode: FileModeStr = "existing_files",
        **kwargs,
    ):
        super().__init__(**kwargs)
        self.set_title(caption)
        self.path_id = path_id
        if extension_filter:
            self.set_extension_filter(extension_filter)
        if path_id is not None and path is None:
            settings = core.Settings()
            path = settings.get(path_id, "")
        if path is not None:
            self.set_directory(path)
        self.set_file_mode(file_mode)
        self.set_accept_mode(mode)

    def _get_map(self):
        maps = super()._get_map()
        maps |= {
            "acceptMode": ACCEPT_MODE,
            "fileMode": FILE_MODE,
            "viewMode": VIEW_MODE,
        }
        return maps

    def set_accept_mode(self, mode: AcceptModeStr | widgets.QFileDialog.AcceptMode):
        """Set accept mode.

        Args:
            mode: accept mode to use
        """
        self.setAcceptMode(ACCEPT_MODE.get_enum_value(mode))

    def get_accept_mode(self) -> AcceptModeStr:
        """Return accept mode.

        Returns:
            accept mode
        """
        return ACCEPT_MODE.inverse[self.acceptMode()]

    def set_view_mode(self, mode: ViewModeStr | widgets.QFileDialog.ViewMode):
        """Set view mode.

        Args:
            mode: view mode to use
        """
        self.setViewMode(VIEW_MODE.get_enum_value(mode))

    def get_view_mode(self) -> ViewModeStr:
        """Return view mode.

        Returns:
            view mode
        """
        return VIEW_MODE.inverse[self.viewMode()]

    def set_label_text(
        self, label: LabelStr | widgets.QFileDialog.DialogLabel, text: str
    ):
        """Set the label text for button label.

        Args:
            label: button to set text for
            text: text to use
        """
        self.setLabelText(LABEL.get_enum_value(label), text)

    def get_label_text(self, label: LabelStr | widgets.QFileDialog.DialogLabel) -> str:
        """Return label text.

        Returns:
            label text
        """
        return self.labelText(LABEL.get_enum_value(label))

    def get_file_mode(self) -> FileModeStr:
        """Return file mode.

        Returns:
            file mode
        """
        return FILE_MODE.inverse[self.fileMode()]

    def set_file_mode(self, mode: FileModeStr | widgets.QFileDialog.FileMode):
        """Set the file mode of the dialog.

        Args:
            mode: mode to use
        """
        self.setFileMode(FILE_MODE.get_enum_value(mode))

    def selected_files(self) -> list[pathlib.Path]:
        return [pathlib.Path(p) for p in self.selectedFiles()]

    def selected_file(self) -> pathlib.Path | None:
        selected = self.selectedFiles()
        return pathlib.Path(selected[0]) if selected else None

    def choose_folder(self) -> list[pathlib.Path] | None:
        self.set_file_mode("directory")
        return self.choose()

    def open_file(self) -> list[pathlib.Path] | None:
        self.set_file_mode("existing_file")
        return self.choose()

    def choose(self) -> list[pathlib.Path] | None:
        result = self.exec()
        if result != self.DialogCode.Accepted:
            return None
        paths = self.selected_files()
        if self.path_id:
            settings = core.Settings()
            folder_path = paths[0].parent
            settings.setValue(self.path_id, str(folder_path))
        return paths

    def set_extension_filter(self, extension_dict: dict[str, list[str]]):
        """Set filter based on given dictionary.

        dict must contain "'name': ['.ext1', '.ext2']" as key-value pairs

        Args:
            extension_dict: filter dictionary
        """
        items = [
            f"{k} ({' '.join(f'*{ext}' for ext in v)})" for k, v in extension_dict.items()
        ]
        filter_str = ";;".join(items)
        self.setNameFilter(filter_str)

    def get_directory(self) -> pathlib.Path:
        """Return current directory.

        returns current directory level as a Pathlib object

        Returns:
            Pathlib object
        """
        return pathlib.Path(self.directory().absolutePath())

    def set_directory(self, path: datatypes.PathType):
        """Set start directory."""
        path = os.fspath(path)
        self.setDirectory(path)

    def set_filter(self, *filters: core.dir.FilterStr):
        flags = core.dir.FILTERS.merge_flags(filters)
        self.setFilter(flags)

    def get_filter(self) -> list[core.dir.FilterStr]:
        return core.dir.FILTERS.get_list(self.filter())


if __name__ == "__main__":
    app = widgets.app()
    widget = FileDialog()
    widget.show()
    app.exec()



from __future__ import annotations

import logging

from typing import Literal

from typing_extensions import Self

from prettyqt import qml
from prettyqt.utils import bidict, get_repr


logger = logging.getLogger()

ErrorTypeStr = Literal["none", "generic", "range", "reference", "syntax", "type", "uri"]

ERROR_TYPES = bidict(
    none=qml.QJSValue.ErrorType(0),
    generic=qml.QJSValue.ErrorType.GenericError,
    range=qml.QJSValue.ErrorType.RangeError,
    reference=qml.QJSValue.ErrorType.ReferenceError,
    syntax=qml.QJSValue.ErrorType.SyntaxError,
    type=qml.QJSValue.ErrorType.TypeError,
    uri=qml.QJSValue.ErrorType.URIError,
)


class JSValue(qml.QJSValue):
    """Acts as a container for Qt/JavaScript data types."""

    def __repr__(self):
        return get_repr(self, self.toVariant())

    def __len__(self):
        return self.property("length").toVariant()

    def __getitem__(self, index: int | str):
        return self.property(index).toVariant()

    def __delitem__(self, index: str):
        self.deleteProperty(index)

    def __setitem__(self, index: int | str, value):
        self.setProperty(index, value)

    def __iter__(self):
        iterator = qml.JSValueIterator(self)
        return iter(list(iterator))

    def __contains__(self, index: str):
        return self.hasProperty(index)

    def __call__(self, *args) -> JSValue:
        result = self.call(args)
        return JSValue(result)

    def get_value(self):
        return self.toVariant()

    def get_error_type(self) -> ErrorTypeStr | None:
        if (error_type := self.errorType()) == qml.QJSValue.ErrorType(0):
            return None
        else:
            return ERROR_TYPES.inverse[error_type]

    @classmethod
    def from_object(cls, obj, jsengine: qml.QJSEngine) -> Self:
        """Convert any python object into a QJSValue (must happen in GUI thread)."""
        match obj:
            case None:
                return cls()
            case list() | tuple():
                length = len(obj)
                array = cls(jsengine.newArray(length))
                for i, v in enumerate(obj):
                    array.setProperty(i, cls.from_object(v, jsengine))
                return array
            case dict():
                array = cls(jsengine.newArray())
                for k, v in obj.items():
                    array.setProperty(k, cls.from_object(v, jsengine))
                return array
            case _:
                try:
                    return cls(obj)
                except TypeError:
                    logger.debug(f"unknown type: {obj}")
                    return cls()


if __name__ == "__main__":
    val = JSValue()



from __future__ import annotations

import logging

import mknodes

from prettyqt import constants, core, itemmodels
from prettyqt.utils import helpers


logger = logging.getLogger(__name__)


class MarkdownModel(itemmodels.TreeModel):
    """An ItemModel for displaying a mknodes tree."""

    class Roles:
        MarkdownRole = constants.USER_ROLE + 5325

    def columnCount(self, index: core.ModelIndex | None = None) -> int:
        return 3

    def data(
        self,
        index: core.ModelIndex,
        role: constants.ItemDataRole = constants.DISPLAY_ROLE,
    ):
        data = self.data_by_index(index).obj
        match role, index.column():
            case constants.DISPLAY_ROLE, 0:
                return repr(data)
            case constants.DISPLAY_ROLE, 1:
                return data.to_markdown().count("\n")
            # case constants.DISPLAY_ROLE, 2:
            #     return data.to_markdown()
            case self.Roles.MarkdownRole, _:
                return data.to_markdown()

    def _fetch_object_children(self, item) -> list[MarkdownModel.TreeItem]:
        return [self.TreeItem(i) for i in item.obj.children]

    def _has_children(self, item) -> bool:
        return len(item.obj.children) > 0


if __name__ == "__main__":
    page = mknodes.MkPage([])
    page += mknodes.MkAdmonition("test")
    page += mknodes.MkTable(data=dict(a=[1, 2], b=["c", "D"]), header="From mapping")
    page += mknodes.MkDocStrings(helpers, header="DocStrings")
    model = MarkdownModel(page)
    print(page.to_markdown())
